#ifndef __DEVICE_H__
#define __DEVICE_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <core/DeviceConfig.h>
#include <core/InputMap.h>
#include <lang/String.h>

namespace core
{
    /**
     * @brief
     * This is an abstract class, where all the specific device context need to extend.
     *  - All the device classes need to extend from this class.
     * 
     * @note
     * All the device classes need to extend from this class.
     */
    class API_STATE Device : public Object
    {
    public:
        /**
         * Window width
         */
		static float32 windowWidth;

		/**
         * Window height
         */
		static float32 windowHeight;

        /**
         * Constructor
         */
        Device();

		/**
         * Destructor
         */
        virtual ~Device();

        /**
         * Create the main device
         *
         * @param device The device configuration
         */
        bool createDevice(DeviceConfig &device);

		/**
         * Initialize the render
         */
        virtual bool run();

        /**
         * Return the time in seconds
         */
		virtual float32 getTimeInSeconds(){return 0;}

        /**
         * Set the data path
         *
         * @param path The data path
         */
		static void setDataPath(lang::String path);

        /**
         * Get the data path
         *
         * @return The data path
         */
		static lang::String &getDataPath();

		/**
         * An auxiliar method to get the real key.
         */
		static uint32 getRealKey(uint32 input);

        /**
         * Return the render object instance
         */
        static render::Render *getRenderInstance();

        /**
         * Return the device object instance
         */
		static Device *getDeviceInstance();

		/**
		 * @see Object::toString()
		 */
		const char *toString(){return "class: Device";}

	protected:
        /**
         * The platform specific key map
         */
		static const int32 keyMap[];

        /**
         * The render device.
         */
        render::Render *_pRenderDevice;

        /**
         * Create the window.
         * 
         * @param renderType The type of render (e.g. RT_OPENGL).
         * @return Returns true if success.
         */
        virtual bool createWindow(DeviceConfig &device);

        /**
         * An auxiliar method to create the render.
         */
        bool createRender(DeviceConfig &device, HWND hWnd);

    private:

        /**
         * The device.
         */
        Device *_pDeviceWindow;

        /**
         * The static render instance.
         */
		static render::Render *_renderInstance;

        /**
         * The static device instance.
         */
		static Device* _deviceInstace;

        /**
         * The data path.
         */
		static lang::String _dataPath;
    };
}

#endif /*DEVICENULL_H_*/
