#ifndef ENGINE_ARRAY_H_
#define ENGINE_ARRAY_H_

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <util/Types.h>
#include <util/Assert.h>

namespace lang
{
	/**
	 * @brief
	 * The Array class.
	 */
    template <class T> class Array
    {
    public:

        /**
         * Constructor
         */
        Array();

        /**
         * Copy Constructor
         * 
         * @param in The array reference
         */
        Array(const Array<T>& in);

        /**
         * Destructor
         */
        virtual ~Array();

        /**
         * Get the length
         * 
         * @return Return the array length
         */
        uint32 getLength() const;

        /**
         * Get the size in bytes
         * 
         * @return Return the array size
         */
        uint32 getSize() const;

        /**
         * Reserve the required amount of space for the array
         * 
         * @param n The desired size of memory
         */
        void reserve(uint32 number);

        /**
         * Free all the space
         */
        void free();

        /**
         * Clear the whole array, but keep the memory space
         */
        void clear();

        /**
         * Add a new element to the end of the array, automatically allocating more space, if needed
         * 
         * @param elem The item to be inserted
         */
        void add(const T elem);

        /**
         * Remove the element in the given position
         * 
         * @param i The index
         */
        void remove(int32 i);

        /**
         * Remove 'n' elements in the given position
         * 
         * @param i The index
         * @param n The number
         */
        void remove(int32 i, int32 n);

        /**
         * Indexing operator returning const
         * 
         * @param i The index
         * @return Return the object const reference
         */
        const T& operator[](int32 i) const;

        /**
         * Indexing operator
         * 
         * @param i The index
         * @return Return the object reference
         */
        T& operator[](int32 i);

        /**
         * Copy Constructor
         * 
         * @param in The object reference
         */
        void operator=(const Array<T>& in);

        /**
         * Adds another array to the end of current array
         * 
         * @param in The object reference
         */
        void operator+=(Array<T>& in);

		/**
		 * @see Object::toString()
		 */
		const char *toString();

    private:
        /**
         * The buffer
         */
        T *_pBuffer;

        /**
         * The array size in bytes
         */
        uint32 _arraySize;

        /**
         * The array length
         */
        uint32 _length;

        /**
         * Resize the array
         */
        void resize();
    };

#include "Array.inl"

} //namespace

#endif
