#ifndef MATH_QUAT_H_
#define MATH_QUAT_H_

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
 
#include <math/Vector3.h>
#include <math/Matrix.h>

namespace math
{
	/**
	 * @brief
 	 * The Quaternion Class.
 	 */
	class API_STATE Quaternion
    {
    public:
        /**
         * Constructor
         */
        Quaternion();

        /**
         * Constructor
         * 
         * @param angle The angle
         * @param axis The vector axis
         */
        Quaternion(float32 angle, const Vector3f &axis);

        /**
         * Copy Constructor
         * 
         * @param rIn The quaternion reference
         */
        Quaternion(const Quaternion &rIn);

        /**
         * Copy Constructor
         * 
         * @param rIn The matrix reference
         */
        Quaternion(const Matrix &rIn);

        /**
         * Normalize the quaternion
         */
        void normalize();

        /**
         * Converts the quaternion into a matrix
         * 
         * @param rMat The matrix reference
         */
        void getMatrix(Matrix &rMat) const;

        /**
         * Get the rotation angle and axis
         * 
         * @param rAngle The angle reference
         * @param rAxis the axis reference
         */
        void getRotate(float32 &rAngle, Vector3f &rAxis) const;

        /**
         * Interpolate two quaternions
         * 
         * @param q1 The first quaternion
         * @param q2 The second quaternion
         * @param interValue The interpolation value
         */
        void lerp(const Quaternion& q1, const Quaternion& q2, float32 interValue);

        /**
         * Multiplication between quaternions
         * 
         * @param q The quaternion reference
         */
        Quaternion operator * (const Quaternion& q);

        /**
         * Addition between quaternions
         * 
         * @param q The quaternion reference
         */
        Quaternion operator + (const Quaternion& q);

		/**
		 * @see Object::toString()
		 */
		const char *toString(){return "class: Quaternion";}

	private:
		float32 x,y,z,w;
    };
}//namespace
#endif
