#ifndef __LIGHT_H__
#define __LIGHT_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <scene/Transformation.h>
#include <math/Vector3.h>
#include <scene/ColorRGBA.h>


namespace scene
{
    /**
     * @brief
     * This is the Light class.
     */
	class API_STATE Light : public Transformation
	{
	public:
		enum E_LIGHT_TYPE
		{
			LT_POINT = 0,
			LT_SPOT,
			LT_DIRECTIONAL
		};

        /**
         * Constructor
         */
		Light();

        /**
         * Destructor
         */
		virtual ~Light() { }

        /**
         * Set the light direction
         *
		 * @param direction The direction value
         */
		void setDirection(const math::Vector3f &direction);

        /**
         * Set the ambient color
         *
		 * @param ambient The ambient value
         */
		void setAmbient(const ColorRGBA &ambient);

        /**
         * Set the diffuse color
         *
		 * @param diffuse The diffuse value
         */
		void setDiffuse(const ColorRGBA &diffuse);

        /**
         * Set the specular color
         *
		 * @param specular The specular value
         */
		void setSpecular(const ColorRGBA &specular);

        /**
         * Set the spot parameters
         *
		 * @param exponent The exponent value
		 * @param cutOff The cutOff value
         */
		void setSpotParams(float32 exponent, float32 cutOff);

        /**
         * Set the light type
         *
		 * @param eType The light type
         */
		void setLightType(E_LIGHT_TYPE eType);

        /**
         * Get the direction
         *
		 * @return The direction
         */
		const math::Vector3f &getDirection();

        /**
         * Get the ambient color
         *
		 * @return The color
         */
		const ColorRGBA &getAmbient();

        /**
         * Get the diffuse color
         *
		 * @return The color
         */
		const ColorRGBA &getDiffuse();

        /**
         * Get the specular color
         *
		 * @return The color
         */
		const ColorRGBA &getSpecular();

        /**
         * Get the light type
         *
		 * @return The light type
         */
		E_LIGHT_TYPE getLightType();

        /**
         * Get the ambient color
         *
		 * @param exponent The exponent value
		 * @param cutOff The cutOff value
         */
		void getSpotParams(float32 &exponent, float32 &cutOff);

        /**
         * Enable the light
         *
		 * @param value Set to enable the light
         */
		void setEnable(bool value);

		/**
         * Verify if the light is enabled
         *
		 * @return True if enabled
         */
		bool isEnabled();

        /**
         * Set the light number
         *
		 * @param number Set to light number
         */
		void setLightNumber(uint32 number);

		/**
         * Get the light number
         *
		 * @return the light number
         */
		uint32 getLightNumber();

		/**
		 * @see Object::toString()
		 */
		const char * toString(){return "class: Light";}

	private:
		/**
         * The light direction
         */
		math::Vector3f _direction;

		/**
         * The ambient color
         */
		ColorRGBA _ambient;

		/**
         * The diffuse color
         */
		ColorRGBA _diffuse;

		/**
         * The specular color
         */
		ColorRGBA _specular;

		/**
         * The exponent value
         */
		float32 _exponent;

		/**
         * The cutoff value
         */
		float32 _cutOff;

		/**
         * Used to enable the light
         */
		bool _enabled;

		/**
         * The light type
         */
		E_LIGHT_TYPE _eType;

		/**
         * The light number
         */
		uint32 _number;
	};

#include <scene/Light.inl>

}

#endif