#ifndef __MESH_H__
#define __MESH_H__

/* Aika
*
* Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/

#include <util/Log.h>
#include <scene/Transformation.h>
#include <lang/String.h>
#include <render/Render.h>
#include <scene/property/MaterialProperty.h>
#include <scene/property/AlphaProperty.h>
#include <scene/property/CullProperty.h>
#include <img/Image.h>

namespace scene
{
    using namespace lang;

    /**
     * @brief
     * This is the Mesh class.
     */
    class API_STATE Mesh : public Transformation
    {
    public:
        /**
         * Constructor
         */
        Mesh();

        /**
         * Destructor
         */
		virtual ~Mesh();

        /**
         * Load the mesh file (AKG file)
         *
		 * @param filename The filename
         */
        void load(const String &filename);

        /**
         * Draw the mesh data
         *
		 * @param pRender The render instance
         */
		void draw(render::Render *pRender);

        /**
         * Set the mesh alpha property
         *
		 * @param alphaProperty The alpha property
         */
		void setMeshAlphaProperty(const AlphaProperty &alphaProperty);

        /**
         * Set the mesh material property
         *
		 * @param materialProperty The material property
         */
		void setMeshMaterialProperty(const MaterialProperty &materialProperty);

		/**
         * Get alpha property
         *
		 * @return The alha property
         */
		const AlphaProperty &getMeshAlphaProperty();

        /**
         * Get material property
         *
		 * @return The material property
         */
		const MaterialProperty &getMeshMaterialProperty();

		/**
		 * @see Object::toString()
		 */
		const char * toString(){return "class: Mesh";}

	private:
		/**
         * The filename
         */
		String _filename;

		/**
         * The texture image object
         */
		img::Image textureImage;

		/**
         * The alpha property
         */
		AlphaProperty _alphaProperty;

		/**
         * The material property
         */
		MaterialProperty _materialProperty;

		/**
         * The cull property
         */
		CullProperty _cullProperty;

		/**
         * The geometry list id
         */
		int32 _listId;

		/**
         * The vertex list
         */
		lang::Array<math::Vector3f> _vertexList;

		/**
         * The normal list
         */
		lang::Array<math::Vector3f> _normalList;

		/**
         * The texture list
         */
		lang::Array<math::Vector3f> _texList;

		/**
         * The image id
         */
		uint32 _imageId;

		/**
         * has texture
         */
		bool _hasTexture;

		/**
         * has texture
         */
		bool _hasMaterial;

		// file types
		/**
         * Open the AKG (aika geometry) file
		 *
		 * @return True if open the file
         */
		bool openAkg();
    };

#include <scene/Mesh.inl>

}//namespace

#endif /*__MESH_H__*/
