#ifndef __NODE_H__
#define __NODE_H__

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <lang/String.h>
#include <util/Object.h>
#include <util/Log.h>
#include <core/Device.h>
#include <math/Vector2.h>
#include <math/Vector3.h>
#include <math/Vector4.h>
#include <scene/ColorRGBA.h>

using namespace render;
using namespace lang;

namespace scene
{
    /**
     * @brief
     * This is the main node class.
     *  - All the node classes need to extend from this class.
     */
	class API_STATE Node : public Object
	{
	public:
		/**
		 * The node type.
		 */
		enum E_TYPE
		{
			T_CAMERA = 0,
			T_LIGHT,
			T_MESH,
			T_SCENE
		};

        /**
         * Constructor
         */
		Node();

        /**
         * Destructor
         */
		virtual ~Node();

        /**
         * Get the size
         *
		 * @return the size
         */
		int32 getSize() const;

        /**
         * Attach the child to node
         *
         * @param pChild The child
		 * @return the size
         */
		int32 attachChild(Node* pChild);

        /**
         * Detach the child to node
         *
         * @param pChild The child
		 * @return the size
         */
		int32 detachChild(Node* pChild);

        /**
         * Set the parent to the node
         *
         * @param pParent The parent
         */
		void setParent(Node* pParent);

        /**
         * Get the parent
         *
		 * @return the parent
         */
		Node *getParent();

        /**
         * Get the specific node by value
         *
		 * @param value The node position
		 * @return the node (if not, return NULL)
         */
		const Node *getNodeAt(uint32 value);

        /**
         * Get the specific node by name
         *
		 * @param value The node name
		 * @return the node (if not, return NULL)
         */
		const Node *getNodeByName(const char *pName);

        /**
         * Update the node
         */
		virtual void update();

        /**
         * Update selected the node
		 *
		 * @note This will only update the specific node
         */
		virtual void updateSelected(){}

        /**
         * Draw
		 *
		 * @param pRender The render instance
         */
		virtual void draw(render::Render *pRender);

        /**
         * Set the node name
		 *
		 * @param pName The name of the node
         */
		void setName(const char* pName);

        /**
         * Get the node name
		 *
		 * @return The name of the node
         */
		const lang::String getName();

        /**
         * Set the node type
		 *
		 * @param eType The type of the node
         */
		void setType(E_TYPE eType);

        /**
         * Get the node type
		 *
		 * @return The type of the node
         */
		E_TYPE getType();

		/**
		 * @see Object::toString()
		 */
		const char *toString(){return "class: Node";}

	private:
        /**
         * The name of node
         */
		lang::String _name;

        /**
         * The node
         */
		lang::Array<Node*> _child;

        /**
         * The node parent
         */
		Node * _pParent;

        /**
         * The number to increase the node list
         */
		int32 _growBy;

        /**
         * The size of node
         */
		int32 _used;

        /**
         * The node type
         */
		E_TYPE _eType;
	};

#include <scene/Node.inl>
}
#endif
