#ifndef LOG_H_
#define LOG_H_

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <stdio.h>
#include <util/Define.h>
#include <util/Object.h>
#include <util/Assert.h>


#if (TRACE_LOG == ON)
#   define TRACE(text) Log __LOG__(text)
#else
#   define TRACE(text)
#endif

/**
 * @brief 
 * Description: Log system.
 *  - All the messages output need to use this class.
 * 
 * @note
 *  There is already an static Log instance, see the Usage.
 *  See the Define.h to know how turn ON the file save.
 * 
 * Usage:
 * @code
 *   01) New instance
 *     Log logging;
 *     logging << "Test";
 * 
 *   02) Static usage
 *     Log::log << "Test"; 
 * 
 * @endcode
 */
class API_STATE Log : public Object
{
private:

    /**
     * The number of instances
     */
    static int32 _count;

    /**
     * The depth for stacktrace
     */
    static int32 _depth;

    /**
     * Method name used in stacktrace
     */
    char *_pMethodName;

public:

    /**
     * Static instance
     */
    static Log log;

	/**
     * Constructor
     */
    Log();

	/**
     * Constructor
     */
    explicit Log(const char* pMethodName);

    /**
     * Destructor
     */
    virtual ~Log();

    /**
     * Operator overload to string
     */
    Log& operator << (char *pText);

    /**
     * Operator overload to signed integer
     */
    Log& operator << (int32 value);

    /**
     * Operator overload to unsigned integer
     */
    Log& operator << (uint32 value);

    /**
     * Operator overload to boolean
     */
    Log& operator << (bool value);

    /**
     * Operator overload to void*
     */
    Log& operator << (void *pObject);

    /**
     * This method is used to output stacktrace.
     * 
     * @param align        align the line.
     * @param action       the action of line.
     * @param methodName   the method name.
     * @see TRACE_LOG definition
     */
    void record(int32 align, const char *pText, const char *pMethodName);

    /**
     * This method is used to output memory allocation statements.
     * 
     * @param pComponent  the component name to be displayed.
     * @param pFile       the caller file.
     * @param line        the caller line.
     * @param ptr         the pointer.
     * @param size        the size.
     * @see MEMORY_LOG definition  
     * @see MEMORY_FILE_LOG definition
     */
    static void memAlloc(char *pComponent, char *pFile, int32 line);

    /**
     * This method is used to output assertions.
     * 
     * @param pErrorType  type of error.
     * @param pFile       the caller file.
     * @param line        the caller line.
     * @see ASSERTIOM_LOG definition
     */
    static void assertMsg(const char* pErrorType, const char* pFile, int32 line);

    /**
	 * Used to close all the logs.
	 */
    static void closeLogs();

	/**
	 * @see Object::toString()
	 */
	const char * toString();
};

#endif /*_LOG_H_*/
