#ifndef OBJECT_H_
#define OBJECT_H_

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <memory.h>
#include <util/Define.h>
#include <util/Types.h>

/**
 * Use this to log properly when we need to create a instance 
 */
#define kNew(classObject) new classObject; Log::memAlloc("NEW", __FILE__, __LINE__);

/**
 * Use this to log properly when we need to delete a instance 
 */
#define kDelete(instObject) delete instObject; if(instObject != NULL){Log::memAlloc("DEL", __FILE__, __LINE__);}

/**
 * @brief  
 * Description: Object declaration.
 *  - All the classes need to extend this class.
 * 
 * @note
 *  The extension attribute must be "public".
 *  All the constructors need to initialize the Object constructor passing the name of the class.
 * 
 * Usage:
 * @code
 *   class Test : public Object
 *   {
 * 		Test()
 * 		{}
 * 
 * 		virtual ~Test()
 * 		{}
 *    
 *      void toString()
 *      {
 *         // print the name of class
 *      }
 *   }; 
 * @endcode
 */
class API_STATE Object
{
private:
    
	/**
	 * The instance counter. Used to know how instances are in the memory.
	 */
	static int32 _instanceCount;

public:
    /**
     * Constructor
     */
    Object();

    /**
     * Destructor
     */
    virtual ~Object();

    /**
     * The overload of NEW
     * 
     * @param size Size of new object
     */
    void *operator new(size_t size);

    /**
     * The overload of DELETE
     * 
     * @param pPointer Pointer to delete
     */
    void operator delete(void *pPointer);

	/**
     * Return the number of instances classes
     */
    static int32 instances();

	/**
     * Indicates whether some other object is "equal to" this one.
     * 
     * @param pObject The object instance
     */
    bool equals(Object *pObject);

	/**
     * Show the pointer of the instance.
     */
	void showPointer();

	/**
	 * Creates and returns a copy of this object.
	 */
	Object *clone();

	/**
     * Returns a string representation of the object.
     */
	virtual const char *toString() = 0;
};

#endif // OBJECT_H_
