#ifndef PTR_H
#define PTR_H

/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

/**
 * @brief 
 * Description: Smart Pointer Declaration.
 *  - Used to make sure that a dynamic object instance will be deleted out of scope automatically.
 * 
 * @note
 *  All the dynamic object instances that need to be released out of scope, need to use smart pointer.
 * 
 * @warning
 *  Don't use when the pointer need to be used out of scope.
 * 
 * Usage:
 * @code
 *   void Function()
 *   {
 * 		Test *test = new Test(); // wrong
 * 
 * 		Ptr<Test> test(new Test); // right use of smart pointer
 *   } 
 * @endcode
 */
template <class T>
class Ptr
{
private:

    /**
     * The pointer to be used
     */
    T* _mObject;

public:

    /**
     * Constructor
     * 
     * Note: Null object 
     */
    Ptr()
    {
        _mObject = 0;
    }

    /**
     * Constructor
     * 
     * Note: Stores the pointer to an object
     * 
     * @param other New pointer 
     */
    Ptr( const Ptr<T>& other )
    {
        _mObject = other.ptr();
    }

    /**
     * Constructor
     * 
     * Note: Stores the pointer to an object
     * 
     * @param New pointer 
     */
    Ptr( T* other )
    {
        _mObject = other;
    }

    /**
     * Destructor 
     */
    ~Ptr()
    {
        if ( _mObject )
            delete _mObject;
    }

    /**
     * Releases old reference if any and stores the new reference
     * 
     * @param other New pointer
     * @return Return new reference
     */
    Ptr<T>& operator=( const Ptr<T>& other )
    {
        if ( _mObject )
            delete _mObject;

        _mObject = other.ptr();
        return *this;
    }

    /**
     * Return reference to the object
     * 
     * @return Return the reference 
     */
    T&	operator* () const
    {
        return *_mObject;
    }

    /**
     * Access member of the object
     * 
     * @return Return the value 
     */
    T*	operator-> () const
    {
        return _mObject;
    }

    /**
     * Returns pointer to the object
     * 
     * @return Return the pointer 
     */
    operator T* () const
    {
        return _mObject;
    }

    /**
     * Returns pointer to the object
     * 
     * @return Return the pointer 
     */
    T* ptr() const
    {
        return _mObject;
    }
};

#endif //PTR_H
