/* Aika
 *
 * Copyright (C) 2007, Erick Jung <erickjung@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */

#include <math.h>
#include <math/Matrix.h>
#include <math/Quaternion.h>
#include <util/Log.h>

namespace math
{
    Matrix::Matrix()
    {
        m[0][0]=m[1][1]=m[2][2]=m[3][3]=1.0f;
        m[0][1]=m[0][2]=m[0][3]=0.0f;
        m[1][0]=m[1][2]=m[1][3]=0.0f;
        m[2][0]=m[2][1]=m[2][3]=0.0f;
        m[3][0]=m[3][1]=m[3][2]=0.0f;
    }

    Matrix::Matrix(const Matrix& in)
    {
        m[0][0]=in.m[0][0];
        m[0][1]=in.m[0][1];
        m[0][2]=in.m[0][2];
        m[0][3]=in.m[0][3];
        m[1][0]=in.m[1][0];
        m[1][1]=in.m[1][1];
        m[1][2]=in.m[1][2];
        m[1][3]=in.m[1][3];
        m[2][0]=in.m[2][0];
        m[2][1]=in.m[2][1];
        m[2][2]=in.m[2][2];
        m[2][3]=in.m[2][3];
        m[3][0]=in.m[3][0];
        m[3][1]=in.m[3][1];
        m[3][2]=in.m[3][2];
        m[3][3]=in.m[3][3];
    }

    Matrix::~Matrix()
    {}

    void Matrix::reset()
    {
        m[0][0]=m[0][1]=m[0][2]=m[0][3]=
            m[1][0]=m[1][1]=m[1][2]=m[1][3]=
            m[2][0]=m[2][1]=m[2][2]=m[2][3]=
            m[3][0]=m[3][1]=m[3][2]=m[3][3]=0.0f;
    }

    void Matrix::loadIdentity()
    {
        m[0][0]=m[1][1]=m[2][2]=m[3][3]=1.0f;
        m[0][1]=m[0][2]=m[0][3]=0.0f;
        m[1][0]=m[1][2]=m[1][3]=0.0f;
        m[2][0]=m[2][1]=m[2][3]=0.0f;
        m[3][0]=m[3][1]=m[3][2]=0.0f;
    }

    void Matrix::setRotation(float32 ang, const Vector3f& dir )
    {
        Vector3f direction = dir;
        direction.normalize();

        //ang*=-AK_PIOVER180;

        float32 fCos=cosf( ang );
        float32 fSin=sinf( ang );

        m[0][0] = ( direction.x * direction.x ) * ( 1.0f - fCos ) + fCos;
        m[0][1] = ( direction.x * direction.y ) * ( 1.0f - fCos ) - (direction.z * fSin);
        m[0][2] = ( direction.x * direction.z ) * ( 1.0f - fCos ) + (direction.y * fSin);

        m[1][0] = ( direction.y * direction.x ) * ( 1.0f - fCos ) + (direction.z * fSin);
        m[1][1] = ( direction.y * direction.y ) * ( 1.0f - fCos ) + fCos ;
        m[1][2] = ( direction.y * direction.z ) * ( 1.0f - fCos ) - (direction.x * fSin);

        m[2][0] = ( direction.z * direction.x ) * ( 1.0f - fCos ) - (direction.y * fSin);
        m[2][1] = ( direction.z * direction.y ) * ( 1.0f - fCos ) + (direction.x * fSin);
        m[2][2] = ( direction.z * direction.z ) * ( 1.0f - fCos ) + fCos;

        m[0][3] = m[1][3] = m[2][3] = 0;
        m[3][0] = m[3][1] = m[3][2] = 0;
        m[3][3] = 1;
    }

    void Matrix::rotate( float32 ang, const Vector3f& dir )
    {
        static Matrix m;
        m.setRotation(ang,dir);
        *this=*this*m;
    }

    void Matrix::lerp(const Matrix& matrix1, const Matrix& matrix2, float32 interValue)
    {
        Quaternion q1(matrix1),q2(matrix2), qt;
        qt.lerp(q1, q2, interValue);
        qt.getMatrix(*this);

        m[3][0]=matrix1.m[3][0]*(1-interValue) + matrix2.m[3][0]*interValue;
        m[3][1]=matrix1.m[3][1]*(1-interValue) + matrix2.m[3][1]*interValue;
        m[3][2]=matrix1.m[3][2]*(1-interValue) + matrix2.m[3][2]*interValue;
    }

    void Matrix::operator=(const Matrix& in)
    {
        m[0][0]=in.m[0][0];
        m[0][1]=in.m[0][1];
        m[0][2]=in.m[0][2];
        m[0][3]=in.m[0][3];
        m[1][0]=in.m[1][0];
        m[1][1]=in.m[1][1];
        m[1][2]=in.m[1][2];
        m[1][3]=in.m[1][3];
        m[2][0]=in.m[2][0];
        m[2][1]=in.m[2][1];
        m[2][2]=in.m[2][2];
        m[2][3]=in.m[2][3];
        m[3][0]=in.m[3][0];
        m[3][1]=in.m[3][1];
        m[3][2]=in.m[3][2];
        m[3][3]=in.m[3][3];
    }

    Matrix Matrix::operator*(const Matrix& in) const
    {
        Matrix out;
        int32 i,j;
        for(i=0; i<4; i++)
            for(j=0; j<4; j++)
            {
                out.m[i][j] = m[i][0]*in.m[0][j] + m[i][1]*in.m[1][j] +
                    m[i][2]*in.m[2][j] + m[i][3]*in.m[3][j];
            }
            return out;
    }
}//namespace
